<?php

/**
 * @file ooyala.api.inc
 *
 * The various api wrapper function in use by the module, the first of which is
 * a class for signing requests to Backlot.
 */

class OoyalaPartnerAPI {
  var $secret;
  var $pcode;

  /**
   * Construct a new OoyalaPartnerAPI object.
   *
   * @param
   *   Optional parameter for the Ooyala API secret key.
   * @param
   *   Optional parameter for the Ooyala API partner code.
   */
  function OoyalaPartnerAPI($secret = NULL, $pcode = NULL) {
    if (!$secret) {
      $secret = variable_get('ooyala_global_secret', '');
    }
    if (!$pcode) {
      $pcode = variable_get('ooyala_global_pcode', '');
    }

    $this->secret = $secret;
    $this->pcode = $pcode;
  }

  /**
   * Return the signed parameters for an array of parameters.
   *
   * @param $params
   *   An array of Ooyala API query parameters.
   *
   * @return
   *   A query string containing the signed parameters.
   */
  function signed_params($params) {

    if (!array_key_exists('expires', $params)) {
      $params['expires'] = REQUEST_TIME + 1000000;  // 15 minutes
    }

    $string_to_sign = $this->secret;
    $param_string = 'pcode=' . $this->pcode;
    $keys = array_keys($params);
    sort($keys);

    foreach ($keys as $key) {

      if (is_array($params[$key])) {
        $params_key = $params[$key][LANGUAGE_NONE][0]['value'];
      }
      else {
        $params_key = $params[$key];
      }

      $string_to_sign .= $key . '=' . $params_key;

      $param_string .= '&' . rawurlencode($key) . '=' . rawurlencode($params_key);
    }

    $digest = hash('sha256', $string_to_sign, TRUE);
    $signature = preg_replace('/=+$/', '', trim(base64_encode($digest)));
    return $param_string . '&signature=' . rawurlencode($signature);

  }
}

/**
 * Check if Ooyala APIs are available.
 *
 * @return boolean
 *   Boolean indicator as to whether or not the api is available.
 */
function ooyala_api_available() {
  static $online;
  if (!isset($online)) {
    $response = drupal_http_request("http://api.ooyala.com/partner/query");
    // Ooyala will return response code 400 if a pcode is not provded, but all
    // we're interested in is the fact that we got a response at all.
    $online = $response->code == 400 || $response->code == 200;
  }
  return $online;
}

/**
 * Retrieve a non-cached list of videos from Ooyala.
 *
 * @param $params
 *   A list of Ooyala parameters on which to base the search.
 * @param $details
 *   Optional. An array to populate with extra information about the results.
 * @return array
 *   An array of video properties keyed by the unique embed code.
 * @see http://www.ooyala.com/support/docs/backlot_api#query
 */
function ooyala_api_video_query($params = array(), &$details = NULL) {
  $videos = array();

  // Query for any new videos that need to be retrieved.
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params($params);
  $response = drupal_http_request("http://api.ooyala.com/partner/query?$request");

  // Verify that we're even setup correctly.
  if ($response->data == 'unknown pcode') {
    drupal_set_message(t('The Ooyala configuration appears to be invalid.'), 'error');
    return FALSE;
  }

  // Populate all the video entries with the results.
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    if (isset($details)) {
      $attributes = (array) $xmldoc->attributes();
      $details = array(
        'total' => $attributes['@attributes']['totalResults'],
        'size' => $attributes['@attributes']['size'],
        'page' => $attributes['@attributes']['pageID'],
      );
    }

    foreach ($xmldoc->children() as $result) {
      $video = (array) $result;

      // Not sure what the stat property is used for yet.
      if (isset($video['stat'])) {
        unset($video['stat']);
      }

      // Convert all items to strings. This ensures that empty items do not
      // return as child SimpleXML objects.
      foreach ($video as $key => $value) {
        if (!in_array($key, array('labels'))) {
          $video[$key] = (string) $value;
        }
      }

      // Convert labels.
      if (isset($video['labels'])) {
        $labels = $video['labels']->children();
        $video['labels'] = array();
        foreach ($labels as $label) {
          // TODO: It would be nice if labels had a label ID here to use as the
          // key. API change required from Ooyala.
          $olid = ooyala_api_label_id($label[0]);
          $video['labels'][$olid] = (string) $label[0];
        }
      }

      $videos[$video['embedCode']] = $video;
    }
  }

  // Even though this query is uncached, since we pulled in this information we
  // might as well put each video into the static cache so we don't need to
  // retrieve it again when using any other API calls that are cached.
  $cache =& ooyala_api_video_cache();
  foreach ($videos as $embedcode => $video) {
    $cache[$embedcode] = $video;
  }

  return $videos;
}

/**
 * Implement the Attribute Update API. This allows for attributes of videos and
 * channels to be updated.
 *
 * @param $embed_code
 *   The embed code of the video or channel to update.
 * @param $params
 *   A list of Ooyala parameters to update values for.
 *
 * @see http://www.ooyala.com/support/docs/backlot_api#attribute
 *
 * @return
 *   TRUE if the update was successful, or FALSE otherwise.
 */
function ooyala_api_edit($params) {
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params($params);
  $response = drupal_http_request("http://api.ooyala.com/partner/edit?$request");

  // Verify that we're even setup correctly.
  if ($response->data == 'unknown pcode') {
    drupal_set_message(t('The Ooyala configuration appears to be invalid.'), 'error');
    return FALSE;
  }

  if ($response->data != 'ok') {
    drupal_set_message(check_plain($response->data), 'error');
    return FALSE;
  }

  return TRUE;
}

/**
 * Function to hold static cache of all queried videos for this request.
 *
 * @param $reset
 *   Boolean indicator as so reset the static cache or not.
 */
function &ooyala_api_video_cache($reset = FALSE) {
  static $videos;
  if (!isset($videos) || $reset) {
    $videos = array();
  }
  return $videos;
}

/**
 * Load an individual video from Ooyala.
 *
 * @param $embedcode
 *   The unique embedcode of the video as defined in Backlot.
 * @return
 *   An array of video properties keyed by embed code.
 */
function ooyala_api_video_load($embedcode) {
  $videos = ooyala_api_video_load_multiple(array($embedcode));
  return isset($videos[$embedcode]) ? $videos[$embedcode] : FALSE;
}

/**
 * Load multiple videos from Ooyala in a single query.
 *
 * @param $embedcodes
 *   An array of videos keyed by embed code.
 * @return
 *   An array of videos and their properties keyed by embed code.
 */
function &ooyala_api_video_load_multiple($embedcodes) {
  $videos =& ooyala_api_video_cache();

  // Build a list of videos to query that we do not yet have retrieved.
  $query_embedcodes = array();
  foreach ($embedcodes as $key => $embedcode) {
    if (!isset($videos[$embedcode])) {
      $query_embedcodes[] = $embedcode;
    }
  }

  // Query for any new videos that need to be retrieved.
  if (!empty($query_embedcodes)) {
    // Default each embedcode to FALSE in case it is not retrieved.
    foreach ($query_embedcodes as $embedcode) {
      $videos[$embedcode] = FALSE;
    }

    // Populate all the video entries with the results.
    $params = array(
      'embedCode' => implode(',', $query_embedcodes),
      'fields' => 'labels',
    );
    $results = ooyala_api_video_query($params);
    if ($results) {
      foreach ($results as $embedcode => $video) {
        $videos[$embedcode] = $video;
      }
    }
  }

  $return = array();
  foreach ($embedcodes as $embedcode) {
    $return[$embedcode] = $videos[$embedcode];
  }

  return $return;
}

/**
 * Load an individual property of a video from Ooyala.
 *
 * @param $embedcode
 *   A string containing the unique embed code of a video.
 * @param $property
 *   The name of the property to retrieve.
 * @return
 *   The individual property specified.
 */
function ooyala_api_video_property($embedcode, $property) {
  $video = ooyala_api_video_load($embedcode);
  return isset($video[$property]) ? $video[$property] : FALSE;
}

/**
 * Remove labels from a video.
 */
function ooyala_api_video_label_remove($embedcode, $labels, $recursive = TRUE) {
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params(
    array(
      'embedCodes' => is_string($embedcode) ? $embedcode : implode(',', $embedcode),
      'labels' => is_string($labels) ? $labels : implode(',', $labels),
      'mode' => 'unassignLabels',
      'includeSublabels' => $recursive ? 'true' : 'false',
    )
  );
  $response = drupal_http_request("http://api.ooyala.com/partner/labels?$request");

  $success = FALSE;
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    $success = ((string) $xmldoc[0]) == 'ok';
  }

  return $success;
}

/**
 * Set labels on a video.
 *
 * @param $embedcode
 *   An Ooyala embed code.
 * @param $labels
 *   An array of labels to set on a video.
 */
function ooyala_api_video_label_add($embedcode, $labels) {
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params(
    array(
      'embedCodes' => is_string($embedcode) ? $embedcode : implode(',', $embedcode),
      'labels' => is_string($labels) ? $labels : implode(',', $labels),
      'mode' => 'assignLabels',
    )
  );
  $response = drupal_http_request("http://api.ooyala.com/partner/labels?$request");

  $success = FALSE;
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    $success = ((string) $xmldoc[0]) == 'ok';
  }

  return $success;
}

/**
 * Retrieve a label ID from Ooyala based on label name.
 */
function ooyala_api_label_id($label_name) {
  // TODO: Swap out for a more efficient API call when Ooyala adds ability to
  // retrieve label name based on ID.
  $label_name = rtrim($label_name, '/');
  $parent_name = substr($label_name, 0, strrpos($label_name, '/'));
  if ($parent_name == '') {
    $parent_name = '<root>';
  }
  $labels = ooyala_api_label_list($parent_name);
  return array_search($label_name, $labels);
}

/**
 * Retrieve a label name from Ooyala based on label ID.
 */
function ooyala_api_label_name($label_id) {
  $labels = ooyala_api_label_list();
  return isset($labels[$label_id]) ? $labels[$label_id] : FALSE;
}

/**
 * Retrieve a list of labels from Ooyala.
 *
 * @param $parent
 *   The parent label under which to retrieve all labels. Use "/" to
 *   retrieve all labels. Use '<root>' to return only the root labels.
 *
 *   Note that Ooyala will accept both a label path (such as /Tags) or an
 *   Ooyala label ID, which is a 32 character hash.
 * @param $reset
 *   Reset the internal cache of labels to retrieve.
 */
function ooyala_api_label_list($parent = '/', $reset = FALSE) {
  static $labels;

  if ($reset || !isset($labels)) {
    $labels = array();
  }

  if (isset($parent) && !isset($labels[$parent])) {
    $labels[$parent] = array();

    // Query for any new videos that need to be retrieved.
    $ooyala_api = new OoyalaPartnerAPI();
    $params = array(
      'mode' => 'listSubLabels',
      'label' => $parent,
    );
    // Currently it is impossible to get just a list of top-level terms, so we
    // have to get the full list.
    if ($parent == '<root>') {
      $params['label'] = '/';
    }

    $request = $ooyala_api->signed_params($params);
    $response = drupal_http_request("http://api.ooyala.com/partner/labels?$request");

    // Populate all the video entries with the results.
    if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
      // Verify that we're even setup correctly.
      if (isset($xmldoc['message']) && 'Invalid pcode.' == (string) $xmldoc['message']) {
        return FALSE;
      }

      foreach ($xmldoc->children() as $result) {
        $row = (array) $result;
        $label = rtrim($row[0], '/');

        // If building a list of root labels, skip all non-root labels.
        $last_slash = strrpos($label, '/');
        if ($parent == '<root>' && $last_slash !== 0) {
          continue;
        }
        // If building a list for a specific label, do not include the root.
        elseif ($parent != '/' && $parent != '<root>' && $last_slash === 0) {
          continue;
        }
        $labels[$parent][$row['@attributes']['id']] = $label;
      }
    }

    // Sort all the labels so that all root items are first, then all second
    // level, etc. This sets us up for width-first operations.
    uasort($labels[$parent], '_ooyala_label_sort');
  }

  return isset($parent) ? $labels[$parent] : TRUE;
}

/**
 * Add a label to the Ooyala label hierarchy.
 *
 * @param $label_name
 *   The name of the label, including the label path.
 *
 * @return
 *   The Ooyala Label ID of the new label.
 */
function ooyala_api_label_add($label_name) {
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params(array(
    'mode' => 'createLabels',
    'labels' => is_string($label_name) ? $label_name : implode(',', $label_name),
  ));
  $response = drupal_http_request("http://api.ooyala.com/partner/labels?$request");

  // Retrieve the label ID for the newly created label.
  $success = FALSE;
  $label_id = FALSE;
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    $success = ((string) $xmldoc[0]) == 'ok';
    if ($success) {
      $label_id = ooyala_api_label_id($label_name);
    }
  }

  return $label_id;
}

/**
 * Rename a label in Ooyala.
 *
 * This function currently only accepts label paths, not label IDs.
 */
function ooyala_api_label_rename($old_label_name, $new_label_name) {
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params(array(
    'mode' => 'renameLabel',
    'oldlabel' => $old_label_name,
    'newlabel' => $new_label_name,
  ));
  $response = drupal_http_request("http://api.ooyala.com/partner/labels?$request");

  $success = FALSE;
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    $success = ((string) $xmldoc[0]) == 'ok';
  }

  return $success;
}

/**
 * Delete a label in Ooyala.
 */
function ooyala_api_label_delete($label_name) {
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params(array(
    'mode' => 'deleteLabels',
    'labels' => $label_name,
  ));
  $response = drupal_http_request("http://api.ooyala.com/partner/labels?$request");

  $success = FALSE;
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    $success = ((string) $xmldoc[0]) == 'ok';
  }

  return $success;
}

/**
 * Load a full channel, listing in all the embedcodes within a channel.
 */
function ooyala_api_channel_load($embedcode) {
  static $channels;

  if (!isset($channels[$embedcode])) {
    $channel = FALSE;

    $ooyala_api = new OoyalaPartnerAPI();
    $request = $ooyala_api->signed_params(
      array(
        'mode' => 'list',
        'channelEmbedCode' => $embedcode,
      )
    );
    $response = drupal_http_request("http://api.ooyala.com/partner/channels?$request");
    if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
      $channel = (array) $xmldoc;

      $videos = array();
      if (isset($channel['item'])) {
        foreach ($channel['item'] as $key => $item) {
          $item = (array) $item;
          $videos[$item['embedCode']] = $item;
        }
      }

      // Put the list of videos in a logical namespace.
      unset($channel['item']);
      $channel['videos'] = $videos;
    }

    $channels[$embedcode] = $channel;
  }

  return $channels[$embedcode];
}

/**
 * Fetch and save a video thumbnail from the Ooyala server.
 *
 * @param $embedcode
 *   A string containing the unique embed code of a video.
 * @return
 *   The path to the thumbnail if we have one, else FALSE.
 */
function ooyala_api_fetch_image($embedcode) {
  $ooyala_api = new OoyalaPartnerAPI();
  $thumb_path = FALSE;
  $request = $ooyala_api->signed_params(
    array(
      'embedCode' => $embedcode,
      'range' => '0-1',
      'resolution' => '1024x768',
    )
  );

  $response = drupal_http_request("http://api.ooyala.com/partner/thumbnails?$request");

  // Successful query case.
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    $thumb_url = (string) $xmldoc->promoThumbnail[0] ;
    $thumb_file = drupal_http_request($thumb_url);
    $thumb_path = ooyala_get_thumbnail_url($embedcode, NULL, TRUE);

    if ($fp = @fopen(file_build_uri($thumb_path), 'w+')) {
      fwrite($fp, $thumb_file->data);
      fclose($fp);
    }
    else {
      ooyala_message('The file @path could not be written, check the permissions on the directory and make sure it is writable.', array('@path' => $thumb_path), WATCHDOG_ERROR);
      $thumb_path = FALSE;
    }
  }
  // Check if this isn't actually a video at all, but a channel. In which case
  // start over and try to retreive the thumbnail of the first video.
  elseif ($channel = ooyala_api_channel_load($embedcode)) {

    if (count($channel['videos'])) {
      $first_embedcode = reset(array_keys($channel['videos']));
      $thumb_path = ooyala_api_fetch_image($first_embedcode);

      // If the first video has a thumbnail path, copy it to serve as the
      // channel thumbnail.
      if ($thumb_path) {
        $channel_thumb_path = str_replace($first_embedcode, $embedcode, $thumb_path);
        copy($thumb_path, $channel_thumb_path);
        $thumb_path = $channel_thumb_path;
      }
    }
    else {
      ooyala_message('The thumbnail for the channel "@embedcode" could not be retrieved from Ooyala. It looks like this embed code is for a channel, but this channel does not contain any videos that have a usable thumbnail.', array('@embedcode' => $embedcode), 'error');
      $thumb_path = FALSE;
    }
  }
  // The most common situation for a 200 but no XML string is an error such
  // as "unknown content" from Ooyala. Errors are returned as plain strings.
  elseif ($response->code == 200) {
    $nodes = ooyala_load_nodes($embedcode);
    $node_link = t('none');
    $node_links = array();
    foreach ($nodes as $node) {
      $node_links[] = l(check_plain($node->title), 'node/' . $node->nid);
    }
    if (count($node_links) > 1) {
      $node_link = theme('item_list', array($node_links));
    }
    elseif (count($node_links) == 1) {
      $node_link = $node_links[0];
    }
    ooyala_message('The thumbnail for the video "@embedcode" could not be retrieved from Ooyala. Ooyala reported the error <code>@error</code>. This video is in use by the following nodes: !node_links', array('@embedcode' => $embedcode, '@error' => $response->data, '!node_links' => $node_link), 'error');
  }
  else {
    ooyala_message('The thumbnail for the video "@embedcode" could not be retrieved from Ooyala because the service returned an HTTP error @code.', array('@embedcode' => $embedcode, '@code' => $response->code), 'error');
  }

  return $thumb_path;
}

/**
 * Fetch automatically generated thumbnails from Ooyala.
 *
 * @param $embedcode
 *   The embed code of the object to fetch thumbnails for.
 * @param $from
 *   Optional parameter with the first thumbnail index to retrieve. Defaults to
 *   0.
 * @param $to
 *   Optional parameter with the last thumbnail index to retrieve. Defaults to
 *   1.
 *
 * @return
 *   An array of thumbnail image URLs, keyed by timestamp, or FALSE if an error
 *   occured.
 */
function ooyala_api_fetch_thumbnails($embedcode, $from = 0, $to = 1) {
  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params(
    array(
      'embedCode' => $embedcode,
      'range' => $from . '-' . $to,
      'resolution' => '1024x768',
    )
  );
  $response = drupal_http_request("http://api.ooyala.com/partner/thumbnails?$request");

  // Successful query case.
  $thumbnails = array();
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    foreach ($xmldoc->thumbnail as $thumbnail) {
      $attributes = $thumbnail->attributes();
      $thumbnails[(string)$attributes->timestamp] = (string)$thumbnail;
    }
    return $thumbnails;
  }

  return FALSE;
}

/**
 * Upload a promotional image for a given embedcode. This function requires
 * cURL to be enabled.
 *
 * @param $embedcode
 *   The embed code of the object to set the promotional image for.
 * @param $path
 *   The path to the image to upload.
 *
 * @return
 *   The URL to the uploaded image, or FALSE if the image was not uploaded.
 */
function ooyala_api_upload_promo_image($embedcode, $path) {
  if (!function_exists('curl_init')) {
    drupal_set_message(t('There was an error uploading the promotional image. Please try again later.'), 'error');
    watchdog('ooyala', "cURL is required to upload promotional images to Ooyala", array(), WATCHDOG_ERROR);
    return FALSE;
  }

  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params(
    array(
      'embed_code' => $embedcode,
    )
  );

  $uri = "http://uploader.ooyala.com/api/upload/preview?$request";

  // We can't use drupal_http_request() as it doesn't handle POSTing files.
  $ch = curl_init("uploader.ooyala.com");
  curl_setopt($ch, CURLOPT_URL, $uri);
  curl_setopt($ch, CURLOPT_POST, TRUE);
  // CURL will automatically read in the file and post it as a multipart POST.
  curl_setopt($ch, CURLOPT_POSTFIELDS, array('file' => '@' . $path));
  // We want to return the data instead of outputting it directly.
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
  // Don't include headers in the output.
  curl_setopt($ch, CURLOPT_HEADER, FALSE);
  // Follow up to 5 redirects.
  curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
  curl_setopt($ch, CURLOPT_MAXREDIRS, 5);

  $response = curl_exec($ch);
  curl_close($ch);

  if ($response && $xmldoc = @simplexml_load_string($response)) {
    return (string)$xmldoc->uploadedFile;
  }

  return FALSE;
}

/**
 * Custom sort function to sort labels, putting root items first.
 */
function _ooyala_label_sort($a, $b) {
  $cmp = substr_count($a, '/') - substr_count($b, '/');
  if ($cmp == 0) {
    $cmp = strcasecmp($a, $b);
  }
  return $cmp;
}
