<?php

/**
 * @file
 * Ooyala Channel APIs.
 */

/**
 * Assign a set of embed codes to the lineup of a channel.
 *
 * @param $channel_embed_code
 *   The embed code of the channel to edit.
 * @param $embed_codes
 *   An array of embed codes to assign to the channel.
 *
 * @return
 *   TRUE if the operation completed, FALSE otherwise.
 */
function ooyala_channels_assign($channel_embed_code, $embed_codes) {
  module_load_include('inc', 'ooyala', 'includes/ooyala.api');

  $params = array(
    'mode' => 'assign',
    'channelEmbedCode' => $channel_embed_code,
    'embedCodes' => implode(",", $embed_codes),
  );

  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params($params);
  $response = drupal_http_request("http://api.ooyala.com/partner/channels?$request");

  // Verify that we're even setup correctly.
  if ($response->data == 'unknown pcode') {
    drupal_set_message(t('The Ooyala configuration appears to be invalid.'), 'error');
    return FALSE;
  }

  if ($response->data != 'ok') {
    drupal_set_message(check_plain($response->data), 'error');
    return FALSE;
  }

  return TRUE;
}

/**
 * Create a channel in the Ooyala backlot.
 *
 * @param $title
 *   The title of the channel to create.
 *
 * @return
 *   The embedCode of the created channel.
 */
function ooyala_channels_create($title) {
  module_load_include('inc', 'ooyala', 'includes/ooyala.api');

  $params = array(
    'title' => $title,
    'mode' => 'create',
  );

  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params($params);
  $response = drupal_http_request("http://api.ooyala.com/partner/channels?$request");

  // Verify that we're even setup correctly.
  if ($response->data == 'unknown pcode') {
    drupal_set_message(t('The Ooyala configuration appears to be invalid.'), 'error');
    return FALSE;
  }

  // Parse the result of creating the channel.
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    return (string)$xmldoc->embedCode;
  }

  return FALSE;
}

/**
 * List the elements of a channel. This includes the details of the channel as
 * well as the items assigned to the channel.
 *
 * @param $channel_embed_code
 *   The embed code of the channel.
 *
 * @return
 *   An array containing the channel details, or FALSE if an error occured.
 */
function ooyala_channels_list($channel_embed_code) {
  module_load_include('inc', 'ooyala', 'includes/ooyala.api');

  $params = array(
    'mode' => 'list',
    'channelEmbedCode' => $channel_embed_code,
  );

  $ooyala_api = new OoyalaPartnerAPI();
  $request = $ooyala_api->signed_params($params);
  $response = drupal_http_request("http://api.ooyala.com/partner/channels?$request");

  // Verify that we're even setup correctly.
  if ($response->data == 'unknown pcode') {
    drupal_set_message(t('The Ooyala configuration appears to be invalid.'), 'error');
    return FALSE;
  }

  // Parse the result of listing the channel.
  if ($response->data && $xmldoc = @simplexml_load_string($response->data)) {
    $channel = array();
    $channel = (array)$xmldoc;

    // Place all items into a lineup property.
    $channel['lineup'] = array();
    foreach ($xmldoc->children() as $child) {
      if ($child->getName() == 'item') {
        $channel['lineup'][] = $child;
      }
    }

    return _ooyala_channels_listing_to_array($channel);
  }

  drupal_set_message(check_plain($response->data), 'error');
  return FALSE;
}

/**
 * Recursively convert an object into an array.
 */
function _ooyala_channels_listing_to_array($object) {
  $object = (array)$object;
  // You can't remove child nodes from a SimpleXML object, so instead of
  // dealing with DOMNodes we unset any 'item' keys we find. Their values
  // should all ready be in $object->lineup.
  if (isset($object['item'])) {
    unset($object['item']);
  }
  foreach ($object as $key => $value) {
    if (is_array($value) || is_object($value)) {
      $object[$key] = _ooyala_channels_listing_to_array($object[$key]);
    }
  }

  return $object;
}

